<?php

/**
 * Forum Object
 */

class UIEForum extends UIEBaseModel {
  protected $table_name = '{f_forums}';

  protected $forum_id;
  protected $title;
  protected $parent_id;
  protected $description;
  protected $weight;
  protected $locked;
  protected $thread_count;
  protected $post_count;
  protected $thread_count_total;
  protected $post_count_total;

  public function __construct() {
  }

  public function Validate() {
    $this->validated = false;
    $validate_mode = $this->GetMode();
    switch ($validate_mode) {
      case UIEMODEL_MODE_ADD:
        if(!$this->title) { $this->AddError('Name not set'); }
        if(!$this->parent_id) { $this->AddError('Parent Forum not set'); }
      break;
      case UIEMODEL_MODE_UPDATE:
        if(!$this->forum_id) { $this->AddError('Forum Id not set'); }
        if(!$this->title) { $this->AddError('Name not set'); }
        if($this->forum_id && $this->forum_id != 1 && !$this->parent_id) { $this->AddError('Parent Forum not set'); }
      break;
      case UIEMODEL_MODE_SEARCH:
      case UIEMODEL_MODE_DELETE:
        if(!$this->forum_id) { $this->AddError('Forum Id not set'); }
      break;
      default:
        $this->AddError('Unknown mode: '.$validate_mode);
      break;
    }
    if($this->NoErrors()) {
      $this->validated = true;
    }
  }

  public function Maintain() {
    if(!$this->IsValidated()) {
      $this->AddError('Not validated');
      return false;
    }

    $success = false;
    $maintain_mode = $this->GetMode();
    switch ($GLOBALS['db_type']) {
      case 'mysql':
      case 'mysqli':
        switch ($maintain_mode) {
          case UIEMODEL_MODE_ADD:
            $success = db_query("INSERT INTO {$this->GetTableName()} (ForumName, ParentForum, ForumDesc, weight, Locked) VALUES ('%s', %d, '%s', %d, %d)", $this->GetTitle(), $this->GetParentId(), $this->GetDescription(), $this->GetWeight(), $this->GetLocked());
            if($success) {
              $query = db_query("SELECT LAST_INSERT_ID() AS id");
              $result = db_fetch_object($query);
              $this->SetForumId($result->id);

              if(variable_get('uieforum_auto_greet', 1)) {
                $vGreetThreadTitle = t('Welcome to ').$this->GetTitle();
                $vGreetThreadContent = t('This is the first thread in your new forum. You can disable this auto-greeting post by disabling the relevant option on the configure page.');
                uieforum_create_thread($vGreetThreadTitle, $vGreetThreadContent, $this->GetForumId());
              }

            }
            break;
          case UIEMODEL_MODE_UPDATE:
            $vQuery = "UPDATE {$this->GetTableName()} SET ForumName='%s', ForumDesc='%s', weight='%d', Locked='%d', ParentForum='%d' WHERE ForumID='%d'";
            $success = db_query($vQuery, $this->GetTitle(), $this->GetDescription(), $this->GetWeight(), $this->GetLocked(), $this->GetParentId(), $this->GetForumId());
            break;
          case UIEMODEL_MODE_DELETE:
            $success = db_query("DELETE FROM {$this->GetTableName()} WHERE ForumID=%d", $this->GetForumId());
            break;
        }
        break;
    }
    return $success ? true : false;
  }

  public function Search() {
    if(!$this->IsValidated()) {
      $this->AddError('Not validated');
      return false;
    }

    $vTempResults = db_fetch_object(db_query("SELECT * FROM {$this->GetTableName()} WHERE ForumID = %d", $this->GetForumId()));
    return $vTempResults;
  }

  public function GetForumId() { return $this->forum_id; }
  public function GetTitle() { return $this->title; }
  public function GetParentId() { return $this->parent_id; }
  public function GetDescription() { return $this->description; }
  public function GetWeight() { return $this->weight; }
  public function GetLocked() { return $this->locked; }
  public function GetThreadCount() { return $this->thread_count; }
  public function GetPostCount() { return $this->post_count; }
  public function GetThreadCountTotal() { return $this->thread_count_total; }
  public function GetPostCountTotal() { return $this->post_count_total; }

  public function SetForumId($pId) { $this->forum_id = $pId; }
  public function SetTitle($pName) { $this->title = $pName; }
  public function SetParentId($pPid) { $this->parent_id = $pPid; }
  public function SetDescription($pDesc) { $this->description = $pDesc; }
  public function SetWeight($pWeight) { $this->weight = $pWeight; }
  public function SetLocked($pLocked) { $this->locked = $pLocked; }
  public function SetThreadCount($pCount) { $this->thread_count = $pCount; }
  public function SetPostCount($pCount) { $this->post_count = $pCount; }
  public function SetThreadCountTotal($pCount) { $this->thread_count_total = $pCount; }
  public function SetPostCountTotal($pCount) { $this->post_count_total = $pCount; }

  public function GetById($pId) {
    $this->SetForumId($pId);
    $vObj = db_fetch_object(db_query("SELECT * FROM {$this->GetTableName()} WHERE ForumID=%d", $this->GetForumId()));
    $this->SetName($vObj->GroupName);
    $this->SetShortName($vObj->GroupShortName);
  }

  public function InitFromResults($pResultsObj) {
    $this->SetForumId($pResultsObj->ForumID);
    $this->SetTitle($pResultsObj->ForumName);
    $this->SetParentId($pResultsObj->ParentForum);
    $this->SetDescription($pResultsObj->ForumDesc);
    $this->SetWeight($pResultsObj->weight);
    $this->SetLocked($pResultsObj->Locked);
    $this->SetThreadCount($pResultsObj->ThreadCount);
    $this->SetPostCount($pResultsObj->PostCount);
    $this->SetThreadCountTotal($pResultsObj->TotalThreadCount);
    $this->SetPostCountTotal($pResultsObj->TotalPostCount);
  }

  public function GetArrayValues() {
    return array(
      'ForumID' => $this->GetForumId()
      , 'ForumName' => $this->GetTitle()
      , 'ParentForum' => $this->GetParentId()
      , 'ForumDesc' => $this->GetDescription()
      , 'weight' => $this->GetWeight()
      , 'Locked' => $this->GetLocked()
      , 'ThreadCount' => $this->GetThreadCount()
      , 'PostCount' => $this->GetPostCount()
      , 'TotalThreadCount' => $this->GetThreadCountTotal()
      , 'TotalPostCount' => $this->GetPostCountTotal()
    );
  }
}
