<?php

/**
 * History Object
 */

class UIEHistory extends UIEBaseModel {
  protected $table_name = '{f_history}';

  protected $uid;
  protected $thread_id;
  protected $timestamp;

  public function __construct() {
  }

  public function Validate() {
    $this->validated = false;
    $validate_mode = $this->GetMode();
    switch ($validate_mode) {
      case UIEMODEL_MODE_ADD:
      case UIEMODEL_MODE_UPDATE:
        if(!$this->uid) { $this->AddError('Uid not set'); }
        if(!$this->thread_id) { $this->AddError('Tid not set'); }
        if(!$this->timestamp) { $this->AddError('Timestamp not set'); }
      break;
      case UIEMODEL_MODE_DELETE:
        if(!$this->timestamp) { $this->AddError('Timestamp not set'); }
      break;
      case UIEMODEL_MODE_SEARCH:
        if(!$this->uid) { $this->AddError('Uid not set'); }
        if(!$this->thread_id) { $this->AddError('Tid not set'); }
      break;
    }
    if($this->NoErrors()) {
      $this->validated = true;
    }
  }

  public function Maintain() {
    if(!$this->IsValidated()) {
      $this->AddError('Not validated');
      return false;
    }

    $vSuccess = false;
    $maintain_mode = $this->GetMode();
    switch ($GLOBALS['db_type']) {
      case 'mysql':
      case 'mysqli':
        switch ($maintain_mode) {
          case UIEMODEL_MODE_ADD:
            $vSuccess = db_query("INSERT INTO {$this->GetTableName()} (uid, ThreadID, timestamp) VALUES (%d, %d, %d)", $this->GetUid(), $this->GetTid(), $this->GetTimestamp());
            break;
          case UIEMODEL_MODE_UPDATE:
            $vSuccess = db_query("UPDATE {$this->GetTableName()} SET timestamp = %d WHERE uid = %d AND ThreadID = %d", $this->GetTimestamp(), $this->GetUid(), $this->GetTid());
            break;
          case UIEMODEL_MODE_DELETE:
            $vSuccess = db_query("DELETE FROM {$this->GetTableName()} WHERE timestamp < %d", $this->GetTimestamp());
            break;
          default:
            $this->AddError('Unknown mode: '.$maintain_mode);
          break;
        }
        break;
    }
    return $vSuccess ? true : false;
  }

  public function Search() {
    if(!$this->IsValidated()) {
      $this->AddError('Not validated');
      return false;
    }

    $vTempResults = db_query("SELECT timestamp FROM {$this->GetTableName()} WHERE uid = %d AND ThreadID = %d", $this->GetUid(), $this->GetTid());
    return $vTempResults;
  }

  public function GetUid() { return $this->uid; }
  public function GetTid() { return $this->thread_id; }
  public function GetTimestamp() { return $this->timestamp; }

  public function SetUid($uid) {
    $this->uid = $uid;
  }

  public function SetTid($thread_id) {
    $this->thread_id = $thread_id;
  }

  public function SetTimestamp($timestamp = false) {
    if(!$timestamp) {
      $timestamp = time();
    }
    $this->timestamp = $timestamp;
  }
}
